using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace Azbest
{
    public class FreeTiles
    {
        public List<RotatedTile> Tiles = new List<RotatedTile>();

        public event MethodInvoker RefreshViewControl = null;

        public FreeTiles()
        {
        }

        public void Draw(Graphics graph, int totalSize)
        {
            try
            {
                int tileSize = totalSize / GameManager.Instance.CurrentBoard.BoardSize;
                graph.Clear(Color.Black);

                int i = 0;
                int iY = 0;
                while (i < Tiles.Count)
                {
                    int iX = 0;
                    while ((i < Tiles.Count) && ((iX + 1) * tileSize <= totalSize))
                    {
                        if (Tiles[i] != null)
                        {
                            TileDrawer.DrawTile(Tiles[i], graph, iX * tileSize, iY * tileSize, tileSize);
                        }
                        ++i;
                        ++iX;
                    }
                    ++iY;
                }
            }
            catch { }
        }

        public void SaveToFile(string fileName)
        {
            File.WriteAllText(fileName, GetTilesDescription());
        }

        public string GetTilesDescription()
        {
            StringBuilder data = new StringBuilder();
            data.AppendLine(Tiles.Count.ToString());
            data.AppendLine(GameManager.Instance.Colors.ToString());
            for (int i = 0; i < Tiles.Count; ++i)
            {
                data.AppendLine(Tiles[i].BaseTile.GetSidesString());
            }
            return data.ToString();
        }

        public void LoadFromFile(string fileName)
        {
            string [] tileInfo = File.ReadAllLines(fileName);
            int tilesCount = int.Parse(tileInfo[0]);
            int sqrt = 1;
            while (sqrt * sqrt < tilesCount)
            {
                ++sqrt;
            }
            GameManager.Instance.CurrentBoard.BoardSize = sqrt;
            GameManager.Instance.Colors = int.Parse(tileInfo[1]);
            Tiles.Clear();
            for (int i = 0; i < tilesCount; ++i)
            {
                string[] sides = tileInfo[i + 2].Split(" ".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                Tiles.Add(new RotatedTile(new Tile(int.Parse(sides[0]), int.Parse(sides[1]), int.Parse(sides[2]), int.Parse(sides[3])),0));
            }            
        }

        public RotatedTile GetTileAt(Point position, int controlWidth)
        {
            int tileSize = controlWidth / GameManager.Instance.CurrentBoard.BoardSize;

            int indexX = position.X / tileSize;
            int indexY = position.Y / tileSize;

            int index = indexX + indexY * GameManager.Instance.CurrentBoard.BoardSize;
            if (index < Tiles.Count)
            {
                return Tiles[index];
            }
            return null;
        }

        public int GetTileIndexAt(Point position, int controlWidth)
        {
            int tileSize = controlWidth / GameManager.Instance.CurrentBoard.BoardSize;

            int indexX = position.X / tileSize;
            int indexY = position.Y / tileSize;

            int index = indexX + indexY * GameManager.Instance.CurrentBoard.BoardSize;
            if (index < Tiles.Count)
            {
                return index;
            }
            return -1;
        }

        internal void RefreshView()
        {
            if (RefreshViewControl != null) RefreshViewControl();
        }
    }
}
