using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Azbest
{
    public class GameManager
    {
        private static GameManager instance = new GameManager();

        private int colors;
        private List<Brush> colorBrushes = new List<Brush>();
        private Board currentBoard = new Board();
        private FreeTiles freeBoardTiles = new FreeTiles();
        private ExternalResolver externalResolverControl = new ExternalResolver();

        public static GameManager Instance
        {
            get { return instance; }
        }

        public ExternalResolver ExternalResolverControl
        {
            get { return externalResolverControl; }
        }

        public FreeTiles FreeBoardTiles
        {
            get { return freeBoardTiles; }            
        }

        public Board CurrentBoard
        {
            get { return currentBoard; }            
        }

        public int Colors
        {
            get { return colors; }
            set { colors = value; UpdateBrushes();  }
        }

        private GameManager()
        {
            colors = 6;
            UpdateBrushes();

        }

        private void UpdateBrushes()
        {
            foreach (Brush brush in colorBrushes)
            {
                brush.Dispose();
            }
            colorBrushes.Clear();
            int numColorDivisions = 1;
            while (numColorDivisions * numColorDivisions * numColorDivisions < colors)
            {
                ++numColorDivisions;
            }
            int singleColorMultiplier = 255 / (numColorDivisions-1);

            for (int i = 0; i < colors; ++i)
            {
                int colorR = (i / numColorDivisions) / numColorDivisions;
                int colorG = (i / numColorDivisions) % numColorDivisions;
                int colorB = i % numColorDivisions;
                colorBrushes.Add(new SolidBrush(Color.FromArgb(singleColorMultiplier * colorR, singleColorMultiplier * colorG, singleColorMultiplier * colorB)));
            }
        }

        public Brush GetBrushForColor(int colorNum)
        {
            return colorBrushes[colorNum];
        }        
    }
}
