using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Azbest
{
    public partial class BoardControl : UserControl
    {
        public BoardControl()
        {
            InitializeComponent();
        }

        private void BoardControl_Paint(object sender, PaintEventArgs e)
        {
            GameManager.Instance.CurrentBoard.Draw(e.Graphics, this.Width);
        }

        private void BoardControl_DragDrop(object sender, DragEventArgs e)
        {
            RotatedTile draggedTile = (RotatedTile)e.Data.GetData(typeof(RotatedTile));
            Point pointInControl = this.PointToClient(new Point(e.X, e.Y));
            if (GameManager.Instance.FreeBoardTiles.Tiles.Contains(draggedTile))
            {
                RotatedTile oldTile = GameManager.Instance.CurrentBoard.GetTileAt(pointInControl, this.Width);
                if (oldTile == null)
                {
                    Point tileIndex = GameManager.Instance.CurrentBoard.GetTileIndexAtControlPoint(pointInControl, this.Width);
                    GameManager.Instance.CurrentBoard.Tiles[tileIndex.X, tileIndex.Y] = draggedTile;
                    int oldIndex = GameManager.Instance.FreeBoardTiles.Tiles.IndexOf(draggedTile);
                    GameManager.Instance.FreeBoardTiles.Tiles.Remove(draggedTile);
                    GameManager.Instance.FreeBoardTiles.Tiles.Insert(oldIndex, null);

                    GameManager.Instance.FreeBoardTiles.RefreshView();
                    Refresh();
                    return;
                }
                Console.Beep();
                return;
            }
            else
            {
                RotatedTile oldTile = GameManager.Instance.CurrentBoard.GetTileAt(pointInControl, this.Width);
                if (oldTile == null)
                {
                    Point tileIndex = GameManager.Instance.CurrentBoard.GetTileIndexAtControlPoint(pointInControl, this.Width);
                    GameManager.Instance.CurrentBoard.Tiles[tileIndex.X, tileIndex.Y] = draggedTile;
                    GameManager.Instance.CurrentBoard.Tiles[lastTakenTileIndex.X, lastTakenTileIndex.Y] = null;
                    Refresh();
                    return;
                }
                else
                {
                    Point tileIndex = GameManager.Instance.CurrentBoard.GetTileIndexAtControlPoint(pointInControl, this.Width);
                    
                    GameManager.Instance.CurrentBoard.Tiles[lastTakenTileIndex.X, lastTakenTileIndex.Y] = oldTile;
                    GameManager.Instance.CurrentBoard.Tiles[tileIndex.X, tileIndex.Y] = draggedTile;
                    Refresh();
                    return;
                }
            }
        }

        private void BoardControl_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }

        private Point lastTakenTileIndex = new Point(0,0);

        private void BoardControl_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                RotatedTile tile = GameManager.Instance.CurrentBoard.GetTileAt(e.Location, this.Width);
                if (tile != null)
                {
                    lastTakenTileIndex = GameManager.Instance.CurrentBoard.GetTileIndexAtControlPoint(e.Location, this.Width);
                    DoDragDrop(tile, DragDropEffects.Move);
                }
            }
            if (e.Button == MouseButtons.Right)
            {
                RotatedTile tile = GameManager.Instance.CurrentBoard.GetTileAt(e.Location, this.Width);
                if (tile != null)
                {
                    tile.Rotate(3);
                    Refresh();
                }
            }
        }

        private void BoardControl_Load(object sender, EventArgs e)
        {
            GameManager.Instance.CurrentBoard.RefreshViewControl += new MethodInvoker(this.Refresh);
        }

    }
}
