using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Windows.Forms;

namespace Azbest
{
    public class CommunicationClient
    {
        TcpClient client = new TcpClient(AddressFamily.InterNetwork);
        NetworkStream stream = null;

        private Tile[] tiles = null;
        private int colors = 0;

        public void Connect()
        {
            client.Connect(IPAddress.Loopback, 2043);
            stream = client.GetStream();

            byte[] readBuffer = new byte[1024 * 1024];

            string alreadyRead = "";
            while (!alreadyRead.Contains("]"))
            {
                int readCount = stream.Read(readBuffer, 0, readBuffer.Length);
                alreadyRead += Encoding.UTF8.GetString(readBuffer, 0, readCount);                
            }
            string[] lines = alreadyRead.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
            if ((lines == null) || (lines.Length == 0))
            {
                MessageBox.Show("Brak danych na wejciu!");
            }
            if (lines[0] != "[")
            {
                MessageBox.Show("Brak nawiasu otwierajacego!");
                return;
            }
            int tileCount = int.Parse(lines[1]);
            colors = int.Parse(lines[2]);
            tiles = new Tile[tileCount];
            for (int i = 0; i < tiles.Length; ++i)
            {
                string[] sideColors = lines[i + 3].Split(" ".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                tiles[i] = new Tile(int.Parse(sideColors[0]), int.Parse(sideColors[1]), int.Parse(sideColors[2]), int.Parse(sideColors[3]));
            }
        }

        public Tile[] Tiles
        {
            get { return tiles; }
        }

        public int Colors
        {
            get { return colors; }
        }

        public void SetBoard(int[,] indices, int[,] rotations)
        {
            try
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("[");
                sb.AppendLine(indices.Length.ToString());
                for (int iY = 0; iY <= indices.GetUpperBound(1); ++iY)
                {
                    for (int iX = 0; iX <= indices.GetUpperBound(0); ++iX)
                    {
                        if (indices[iX, iY] > -1)
                        {
                            sb.Append(indices[iX, iY].ToString());
                            sb.AppendLine(" " + rotations[iX, iY].ToString());
                        }
                        else
                        {
                            sb.AppendLine(indices[iX, iY].ToString());
                        }
                    }
                }
                sb.AppendLine("]");
                byte[] writableString = Encoding.UTF8.GetBytes(sb.ToString());
                stream.Write(writableString, 0, writableString.Length);
            }
            catch { }
        }

        public void Disconnect()
        {
            byte[] endMessage = Encoding.UTF8.GetBytes("[" + Environment.NewLine + "-1" + Environment.NewLine + "]");
            stream.Write(endMessage, 0, endMessage.Length);
            stream.Close(10000);
            stream = null;            
            client.Close();
        }
    }
}
