#define LOG_TO_FILE
#define USE_SINGLE_PIECE_REDUCTION
//#define USE_TOO_LONG_SEARCH_REDUCTION
//#define USE_RANDOMIZED_SEQUENCE
#define USE_UNIQUENESS_SEQUENCE
//#define USE_UNIQUENESS_SEQUENCE_REVERSED

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.IO;

namespace Azbest
{
    public class GameManager
    {        
        private static GameManager instance = new GameManager();
        private int[,] sendingIndices = null;
        private int[,] sendingRotations = null;
        
        public GameTile [,] BoardTiles;

        public GameTile[] Tiles;

        public int BoardSize;

        public int Colors;

        public CommunicationClient CommClient = new CommunicationClient();

        public TileFinder TileFinder = null;

        public static GameManager Instance
        {
            get { return instance; }
        }

        public void SetTiles(Tile [] tiles, int colors)
        {
            Colors = colors;
            BoardTiles = new GameTile[BoardSize+2, BoardSize+2]; // dla stranikw
            sendingIndices = new int[BoardSize, BoardSize];
            sendingRotations = new int[BoardSize, BoardSize];
            for (int iY = 0; iY < BoardSize+2; ++iY)
            {
                for (int iX = 0; iX < BoardSize+2; ++iX)
                {
                    BoardTiles[iX, iY] = GameTile.EmptyTile;
                    if ((iX == 0) || (iY == 0) || (iX == BoardSize + 1) || (iY == BoardSize + 1))
                    {
                        BoardTiles[iX, iY] = GameTile.BorderTile;
                    }                    
                }
            }
            Tiles = new GameTile[tiles.Length];
            for (int i =0; i < tiles.Length; ++i)
            {
                Tiles[i] = new GameTile(tiles[i], 0, i, 0);
            }
            TileFinder = new TileFinder(Tiles, colors);
        }

        private StreamWriter logFile = null;
        private int recursionDepth = 0;
        private int[] recursionTable = null;
        private List<BoardSocket> emptySockets = null;

        public void StartLogging()
        {
#if LOG_TO_FILE
            logFile = File.CreateText("log_s" + BoardSize.ToString() + "_c" + Colors.ToString() + ".txt");
            recursionTable = new int[BoardSize * BoardSize+1];
            recursionDepth = 0;
            for (int i = 0; i < recursionTable.Length; ++i)
            {
                recursionTable[i] = 0;
            }
#endif
        }

        public void Log(string text)
        {
#if LOG_TO_FILE
            logFile.WriteLine(text);
#endif
        }

        public void StopLogging()
        {
#if LOG_TO_FILE
            logFile.Close();
#endif
        }

        public enum SearchResult
        {
            FoundSolution = 0,
            NoMatches = 1,
            PerformSinglePieceReduction = 2,
            TooLongSearchReduction = 3
        }



#if USE_SINGLE_PIECE_REDUCTION
        private int reducedSinglePatternLeft = 0;
        private int reducedSinglePatternUp = 0;
        private int reducedSinglePatternRight = 0;
        private int reducedSinglePatternDown = 0;
        private BoardSocket reducedSinglePosition = new BoardSocket();
        private List<GameTile> reducedSingleLastPlacedTiles = new List<GameTile>();
#endif

#if USE_TOO_LONG_SEARCH_REDUCTION
        private int reduceTimeByLevels = 250;
        private int recursionsWithoutBreakthrough;
        private int maxRecursionsWithoutBreakthrough = 10; // 50 dziala dobrze dla 9x9 przy unique_sort
        private int deepestLevel;
        private int returnToLevel;
#endif

        public void FindSolutionForSockets(List<BoardSocket> socketsToFill)
        {
            emptySockets = socketsToFill;
#if USE_TOO_LONG_SEARCH_REDUCTION
            
            bool foundSolution = false;
            while (!foundSolution)
            {
                recursionsWithoutBreakthrough = 0;
                deepestLevel = 0;
                SearchResult procRes = FindSolutionForSocketsInternal(emptySockets.Count - 1);
                foundSolution = procRes == SearchResult.FoundSolution;
                
                if (!foundSolution)
                {
                    if (reduceTimeByLevels > 20)
                    {
                        reduceTimeByLevels = (int)(0.66f * reduceTimeByLevels);
                    }
                    else
                    {
                        --reduceTimeByLevels;
                    }
#if LOG_TO_FILE
                    logFile.WriteLine("Nie znalazlem rozwiazania w trakcie " + maxRecursionsWithoutBreakthrough.ToString() + " prob. Od teraz probuje redukowac poziom o " + reduceTimeByLevels.ToString() + ".");
#endif
                }
            }
#else
            FindSolutionForSocketsInternal(emptySockets.Count-1);
#endif

        }

        public SearchResult FindSolutionForSocketsInternal(int socketIndex)
        {
            int patternLeft;
            int patternUp;
            int patternRight;
            int patternDown;
            List<GameTile> matchingTiles;

#if LOG_TO_FILE
            ++recursionTable[recursionDepth++];
#else
            recursionDepth++;
#endif
#if USE_TOO_LONG_SEARCH_REDUCTION
            ++recursionsWithoutBreakthrough;
            if (recursionsWithoutBreakthrough > maxRecursionsWithoutBreakthrough)
            {
                recursionsWithoutBreakthrough = 0;
                --recursionDepth;
                deepestLevel = 0;
                //Console.Beep();
                return SearchResult.TooLongSearchReduction;
            }
            if (recursionDepth > deepestLevel)
            {
                deepestLevel = recursionDepth;
                recursionsWithoutBreakthrough = 0;
                returnToLevel = Math.Max(1, recursionDepth - reduceTimeByLevels);
            }
#endif
            if (socketIndex >= 0)
            {
                BoardSocket socket = emptySockets[socketIndex];
                patternLeft = BoardTiles[socket.X - 1, socket.Y].RightColor;
                patternUp = BoardTiles[socket.X, socket.Y - 1].BottomColor;
                patternRight = BoardTiles[socket.X + 1, socket.Y].LeftColor;
                patternDown = BoardTiles[socket.X, socket.Y + 1].TopColor;
                matchingTiles = TileFinder.GetMatchingTiles(patternLeft, patternUp, patternRight, patternDown);
                ReorganizeTiles(matchingTiles);

                for (int tileIndex = 0; tileIndex < matchingTiles.Count; ++tileIndex)
                {
                    GameTile tile = matchingTiles[tileIndex];
                    if (TileFinder.PlaceTileIfPossible(tile, patternLeft, patternUp, patternRight, patternDown))
                    {
                        TileFinder.RemoveFreeTile(tile);
                        BoardTiles[socket.X, socket.Y] = tile;

                        if (Control.ModifierKeys == Keys.Shift)
                        {
                            SendCurrentDataToVisualization();
                        }
                        SearchResult result = FindSolutionForSocketsInternal(socketIndex-1);

                        if (result == SearchResult.FoundSolution)
                        { // mamy rozwiazanie
#if LOG_TO_FILE
                            logFile.WriteLine("Na poziomie " + recursionDepth.ToString() + " wybralem plytke numer " + (matchingTiles.IndexOf(tile) + 1).ToString() + " z " + matchingTiles.Count.ToString() + " dostepnych. Spedziwszy na tym poziomie " + recursionTable[recursionDepth].ToString());
#endif
                            --recursionDepth;
                            return SearchResult.FoundSolution;
                        }

                        TileFinder.TileIsRemovedFromBoard(tile, patternLeft, patternUp, patternRight, patternDown);
                        BoardTiles[socket.X, socket.Y] = GameTile.EmptyTile;
                        TileFinder.AddFreeTile(tile);

#if USE_TOO_LONG_SEARCH_REDUCTION
                        if (result == SearchResult.TooLongSearchReduction)
                        {
                            if (returnToLevel < recursionDepth)
                            {
                                --recursionDepth;
                                return SearchResult.TooLongSearchReduction;
                            }
                        }
#endif

#if USE_SINGLE_PIECE_REDUCTION
                        if (result == SearchResult.PerformSinglePieceReduction)
                        {
                            if (!reducedSinglePosition.IsNeighbour(ref socket))
                            {   // jeli nasza pozycja nie ssiaduje bezporednio z t popsut
                                BoardSocket prevSocket = emptySockets[socketIndex];
                                if (!BoardTiles[prevSocket.X, prevSocket.Y].AnyRotationMatchesPattern(reducedSinglePatternLeft, reducedSinglePatternUp, reducedSinglePatternRight, reducedSinglePatternDown))
                                { 
                                    --recursionDepth;
                                    return SearchResult.PerformSinglePieceReduction;                                    
                                }
                                // jeli znajdziemy zamiennik, to zostanie on uwolniony, bo zamiast niego uyjemy innego
                            }
                        }
#endif
                    }
                }
                --recursionDepth;

#if USE_SINGLE_PIECE_REDUCTION
                if (matchingTiles.Count == 0)
                {
                    reducedSinglePatternLeft = patternLeft;
                    reducedSinglePatternUp = patternUp;
                    reducedSinglePatternRight = patternRight;
                    reducedSinglePatternDown = patternDown;
                    reducedSinglePosition = socket;

                    return SearchResult.PerformSinglePieceReduction;
                }
                return SearchResult.NoMatches;
#else
                return SearchResult.NoMatches;
#endif
            }
            else
            {   // emptySockets == 0
                --recursionDepth;
                return SearchResult.FoundSolution;
            }
        }

        private GameManager()
        {
        }

        public void ReorganizeTiles(List<GameTile> matchingTiles)
        {
#if USE_UNIQUENESS_SEQUENCE
            TileFinder.UpdateTileUniqueness(matchingTiles);
            matchingTiles.Sort(new GameTileUniquenessComparer());
#endif
#if  USE_UNIQUENESS_SEQUENCE_REVERSED
            TileFinder.UpdateTileUniqueness(matchingTiles);
            matchingTiles.Sort(new GameTileUniquenessRevComparer());
#endif
#if USE_RANDOMIZED_SEQUENCE
            Random rand = new Random();
            int randIndex;
            for (int i = 0; i < matchingTiles.Count; ++i)
            {
                GameTile tmp = matchingTiles[i];
                randIndex = rand.Next(matchingTiles.Count);
                matchingTiles[i] = matchingTiles[randIndex];
                matchingTiles[randIndex] = tmp;                
            }
#endif
        }

        public void SendCurrentDataToVisualization()
        {            
            for (int iY = 0; iY < BoardSize; ++iY)
            {
                for (int iX = 0; iX < BoardSize; ++iX)
                {
                    if (BoardTiles[iX+1, iY+1] == GameTile.EmptyTile)
                    {
                        sendingIndices[iX, iY] = -1;
                        sendingRotations[iX, iY] = -1;
                    }
                    else
                    {
                        sendingIndices[iX, iY] = BoardTiles[iX+1, iY+1].Index;
                        sendingRotations[iX, iY] = BoardTiles[iX+1, iY+1].Rotation;
                    }
                }
            }
            CommClient.SetBoard(sendingIndices, sendingRotations);
        }
    }
}
