using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Azbest
{
    public class TomSolverBrute
    {

        private List<BoardSocket> CreateSocketsSortedByDistanceToCenter(int boardSize)
        {
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);
            for (int iY = 0; iY < boardSize; ++iY)
            {
                for (int iX = 0; iX < boardSize; ++iX)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                }
            }
            emptySockets.Sort(new BoardSocketCenterDistSorter());
            return emptySockets;
        }

        private List<BoardSocket> CreateSocketsBorderFirstSortedCenter(int boardSize)
        {
            // najpierw rogi, potem obwdka, na koniec wok rodka
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);
            for (int iY = 1; iY < boardSize - 1; ++iY)
            {
                for (int iX = 1; iX < boardSize - 1; ++iX)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                }
            }
            emptySockets.Sort(new BoardSocketCenterDistSorter());

            List<BoardSocket> borderSockets = new List<BoardSocket>();
            borderSockets.Add(new BoardSocket(1, 1));
            borderSockets.Add(new BoardSocket(boardSize, 1));
            borderSockets.Add(new BoardSocket(boardSize, boardSize));
            borderSockets.Add(new BoardSocket(1, boardSize));
            for (int iX = 1; iX < boardSize - 1; ++iX)
            {
                borderSockets.Add(new BoardSocket(iX + 1, 1));
            }
            for (int iY = 1; iY < boardSize - 1; ++iY)
            {
                borderSockets.Add(new BoardSocket(boardSize, iY + 1));
            }
            for (int iX = boardSize - 2; iX >= 1; --iX)
            {
                borderSockets.Add(new BoardSocket(iX + 1, boardSize));
            }
            for (int iY = boardSize - 2; iY >= 1; --iY)
            {
                borderSockets.Add(new BoardSocket(1, iY + 1));
            }
            borderSockets.Reverse();
            emptySockets.AddRange(borderSockets);
            return emptySockets;
        }

        private List<BoardSocket> CreateSocketDiagonalSnake(int boardSize)
        {
            List<BoardSocket> emptySockets = new List<BoardSocket>();
            int iX = 0;
            int iY = 0;

            emptySockets.Add(new BoardSocket(iX+1, iY+1));
            while (true)
            {
                if (iX < boardSize - 1)
                {
                    ++iX;
                    emptySockets.Add(new BoardSocket(iX+1, iY+1));
                }
                else
                {
                    ++iY;
                    emptySockets.Add(new BoardSocket(iX+1, iY+1));
                }
                while ((iX > 0) && (iY < boardSize-1))
                {
                    --iX;
                    ++iY;
                    emptySockets.Add(new BoardSocket(iX+1, iY+1));
                }

                if (iY < boardSize - 1)
                {
                    ++iY;
                    emptySockets.Add(new BoardSocket(iX+1, iY+1));
                }
                else
                {
                    ++iX;
                    emptySockets.Add(new BoardSocket(iX+1, iY+1));
                }
                while ((iX < boardSize - 1) && (iY > 0))
                {
                    --iY;
                    ++iX;
                    emptySockets.Add(new BoardSocket(iX+1, iY+1));
                }
                if ((iX == boardSize - 1) && (iY == boardSize - 1)) break;
            }
            emptySockets.Reverse();
            return emptySockets;
        }

        private List<BoardSocket> CreateSocketsByChessboard(int boardSize)
        {
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);
            for (int iY = 0; iY < boardSize; ++iY)
            {
                for (int iX = 0; iX < boardSize; ++iX)
                {
                    if ((iX + iY) % 2 == 0)
                    {
                        emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                    }
                }
            }
            for (int iY = 0; iY < boardSize; ++iY)
            {
                for (int iX = 0; iX < boardSize; ++iX)
                {
                    if ((iX + iY) % 2 == 1)
                    {
                        emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                    }
                }
            }
            emptySockets.Reverse();
            return emptySockets;
        }

        private List<BoardSocket> CreateSocketsByBlocks(int boardSize)
        {
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);
            int iX, iY;
            for (int internalSize = 0; internalSize < boardSize; ++internalSize)
            {
                iY = internalSize;
                for (iX = 0; iX < internalSize; ++iX)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                }
                iX = internalSize;
                for (iY = 0; iY < internalSize; ++iY)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                }
                emptySockets.Add(new BoardSocket(internalSize + 1, internalSize + 1));
            }
            emptySockets.Reverse();
            return emptySockets;
        }        

        private List<BoardSocket> CreateSocketsByTwoIntersectingBlocks(int boardSize)
        {            
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);

            List<BoardSocket> leftUp = CreateSocketsByBlocks(boardSize);
            List<BoardSocket> modList = CreateSocketsByBlocks(boardSize);
            leftUp.Reverse();
            modList.Reverse();

            List<BoardSocket> rightDown = new List<BoardSocket>(boardSize * boardSize);
            modList.ForEach(new Action<BoardSocket>(delegate(BoardSocket socket)
                {
                    socket.X = boardSize + 1 - socket.X;
                    socket.Y = boardSize + 1 - socket.Y;
                    rightDown.Add(socket);
                }
                ));
            for (int i = 0; i < boardSize * boardSize; ++i)
            {
                if (!emptySockets.Contains(leftUp[i]))
                {
                    emptySockets.Add(leftUp[i]);                    
                }
                if (!emptySockets.Contains(rightDown[i]))
                {
                    emptySockets.Add(rightDown[i]);
                }
            }
            emptySockets.Reverse();
            
            return emptySockets;
        }

        private List<BoardSocket> CreateSocketsByCrossAndFilling(int boardSize)
        {
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);
            int halfSize = boardSize / 2;
            

            for (int iX = 0; iX < boardSize; ++iX)
            {
                emptySockets.Add(new BoardSocket(iX + 1, halfSize + 1));
                if ((boardSize & 1) == 0)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, halfSize));
                }
            }

            for (int iY = 0; iY < boardSize; ++iY)
            {
                if (iY+1 == halfSize+1) continue;
                if (((boardSize & 1) == 0) && (iY+1 == halfSize)) continue;
                emptySockets.Add(new BoardSocket(halfSize + 1, iY + 1));
                if ((boardSize & 1) == 0)
                {
                    emptySockets.Add(new BoardSocket(halfSize, iY + 1));
                }
            }

            int fillingSize = halfSize;
            if ((boardSize & 1) == 0)
            {
                --fillingSize;
            }
            int nextFillingStart = (boardSize - fillingSize);

            List<BoardSocket> modList = CreateSocketsByBlocks(fillingSize);
            modList.Reverse();

            List<BoardSocket> leftUp = new List<BoardSocket>(fillingSize * fillingSize);
            modList.ForEach(new Action<BoardSocket>(delegate(BoardSocket socket)
                {
                    leftUp.Add(socket);
                }
                ));
            List<BoardSocket> rightUp = new List<BoardSocket>(fillingSize * fillingSize);
            modList.ForEach(new Action<BoardSocket>(delegate(BoardSocket socket)
                {
                    socket.X = nextFillingStart + socket.X;
                    rightUp.Add(socket);
                }
                ));
            List<BoardSocket> leftDown = new List<BoardSocket>(fillingSize * fillingSize);
            modList.ForEach(new Action<BoardSocket>(delegate(BoardSocket socket)
                {
                    socket.Y = nextFillingStart + socket.Y;
                    leftDown.Add(socket);
                }
                ));
            List<BoardSocket> rightDown = new List<BoardSocket>(fillingSize * fillingSize);
            modList.ForEach(new Action<BoardSocket>(delegate(BoardSocket socket)
                {
                    socket.X = nextFillingStart + socket.X;
                    socket.Y = nextFillingStart + socket.Y;
                    rightDown.Add(socket);
                }
                ));
            
            emptySockets.AddRange(leftUp);
            emptySockets.AddRange(rightUp);
            emptySockets.AddRange(leftDown);
            emptySockets.AddRange(rightDown);
            
            emptySockets.Reverse();
            
            return emptySockets;
        }


        private List<BoardSocket> CreateSocketsByArrows(int boardSize)
        {
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);
            int iX, iY;
            for (int internalSize = 0; internalSize < boardSize; ++internalSize)
            {
                emptySockets.Add(new BoardSocket(internalSize + 1, internalSize + 1));
                iY = internalSize;
                for (iX = internalSize - 1; iX >=0 ; --iX)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                }
                iX = internalSize;
                for (iY = internalSize - 1; iY >=0 ; --iY)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                }                
            }
            emptySockets.Reverse();
            return emptySockets;
        }

        private List<BoardSocket> CreateSocketsByThickDiagonalLine(int boardSize)
        {
            List<BoardSocket> emptySockets = new List<BoardSocket>(boardSize * boardSize);
            int iX =0;
            int iY =0;

            // thick line
            while (iX < boardSize && iY < boardSize)
            {
                emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                if ((iX + iY) % 2 == 0)
                {
                    ++iX;
                }
                else
                {
                    ++iY;
                }
            }
            for (int start = 0; start < boardSize; ++start)
            {
                iX = start + 2;
                iY = 0;
                while (iX < boardSize && iY < boardSize)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                    ++iX;                    
                    ++iY;
                }
                iX = 0;
                iY = start + 1;
                while (iX < boardSize && iY < boardSize)
                {
                    emptySockets.Add(new BoardSocket(iX + 1, iY + 1));
                    ++iX;
                    ++iY;
                }
            }
            emptySockets.Reverse();
            return emptySockets;
        }

        public void Solve()
        {
            Random rand = new Random();
            bool success = false;

            while (!success)
            {
                try
                {
                    GameManager.Instance.CommClient.Connect();
                    success = true;
                }
                catch
                {
                    Console.WriteLine("Nie udao si poczy z serwerem. Prbuj dalej");
                    Thread.Sleep(3000);
                }
            }

            int boardSize = 1;
            while (boardSize * boardSize < GameManager.Instance.CommClient.Tiles.Length)
            {
                ++boardSize;
            }

            GameManager.Instance.BoardSize = boardSize;
            GameManager.Instance.SetTiles(GameManager.Instance.CommClient.Tiles, GameManager.Instance.CommClient.Colors);
            GameManager.Instance.StartLogging();            
            
            //List<BoardSocket> emptySockets = CreateSocketsBorderFirstSortedCenter(boardSize);
            //List<BoardSocket> emptySockets = CreateSocketDiagonalSnake(boardSize);
            //List<BoardSocket> emptySockets = CreateSocketsByChessboard(boardSize);
            //List<BoardSocket> emptySockets = CreateSocketsByCrossAndFilling(boardSize);
            List<BoardSocket> emptySockets = CreateSocketsByBlocks(boardSize);
            

            DateTime startTime = DateTime.Now;
            GameManager.Instance.FindSolutionForSockets(emptySockets);            

            GameManager.Instance.SendCurrentDataToVisualization();
            
            for (int i = 0; i < 15; ++i)
            {
                Console.Beep(50 + i * 100, 10);
            }
            string summaryMessage = "Rozwiazywanie planszy " + boardSize.ToString() + "x" + boardSize.ToString() + " z " + GameManager.Instance.Colors.ToString() + " kolorami zajelo " + (DateTime.Now - startTime).TotalSeconds.ToString() + " sekund.";
            GameManager.Instance.Log(summaryMessage);
            GameManager.Instance.StopLogging();
            MessageBox.Show(summaryMessage);
            GameManager.Instance.SendCurrentDataToVisualization();

            GameManager.Instance.CommClient.Disconnect();
        }        
    }
}
